__all__ = ["VERSION"]

try:
    # file generated by setuptools_scm
    from nichtparasoup.__version__ import version
    VERSION = str(version or 'UNKNOWN.falsy')
except ImportError:  # pragma: no cover
    VERSION = 'UNKNOWN.import'
